// This program is copyright VUW.
// You are granted permission to use it to construct your answer to a COMP102 Lab Assessment.
// You may not distribute it in any other way without permission.

/* Code for COMP102 - 2020T3, Lab Assessment 3
 * Name:
 * Username:
 * ID:
 */

import ecs100.*;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import java.io.File;
import java.awt.Color;

/**
 * Question 3. ArrayLists of Numbers                     [23 marks]
 *
 * Complete the findMovingAverage(...) method.
 * 
 * Suppose you have a sequence of numbers representing the end-of-day stock price of a NZ
 * company over a two week period. This data is shown in the handout.
 *
 * Notice that the red line jumps around a lot, giving the graph a sharp, random-seeming
 *  appearance. In order to smooth this data out, analysts often calulate a moving average, as
 *  demonstrated by the blue line. Each average is taken from three data points; for example,
 *  the first average ($4.80) is the average of the first three data points ($4.84, $4.83, and $4.74).
 * 
 * Complete the findMovingAverage(...) method so that it will return a list of smoothed values
 *  by finding the three-point moving average of the numbers.
 *  
 * For example, if your data set is the numbers 4, 8, 2, 1, 6, findMovingAverages(...) would:
 *   1. Find the average of the first three numbers: (4+8+2)/3 = 4.67
 *   2. Move along one space: (8+2+1)/3 = 3.67
 *   3. Move along one space: (2+1+6)/3 = 3
 *   4. Return the list:
 *          [4.67, 3.67, 3]
 *
 * There are two ways to run the program:
 * - You can press the ’Run’ button to load the data from the q3data.txt file.
 * - You can enter your own data (separated by spaces) into the text-field on the left of the
 *   UI, and press enter.
 */
public class Program3
{
    /**
     * Calculates the moving average of a list of doubles
     *
     * @param nums The list of doubles to process
     *
     * @return an ArrayList of doubles containing the averages
     */
    public ArrayList <Double> findMovingAverage(ArrayList<Double> nums){
        ArrayList<Double> averages = new ArrayList<>();

        /*# YOUR CODE HERE */

        return averages;
    }

    /**
     * Loads the data in q3data.txt and processes it.
     */
    private void doRun() {
        try {
            String input = Files.readString(Path.of("q3data.txt"));
            doFindMovingAverage(input);
        } catch (IOException e) {
            UI.println("Error reading file: " + e.getMessage());
        }
    }

    /**
     * Processes a list of numbers stored as a string.
     * Used to handle user input via the textfield in the UI.
     */
    private void doFindMovingAverage(String s) {
        ArrayList<Double> nums = new ArrayList<>();
        Scanner scanner = new Scanner(s);
        while(scanner.hasNextDouble()) {
            nums.add(scanner.nextDouble());
        }
        ArrayList<Double> averages = findMovingAverage(nums);
        UI.print("Input: ");
        UI.println(nums.toString());
        UI.println("Moving Average:");
        for(Double d : averages){
            UI.printf(" %.2f%n", d);
        }
        UI.println();
        this.display(nums, averages);
    }

    /*********************************************
     * YOU CAN IGNORE EVERYTHING BELOW THIS LINE *
     *********************************************/ 
    private final double GRAPH_LEFT = 50;
    private final double GRAPH_BASELINE = 350;
    private final double GRAPH_WIDTH = 400;
    private final double GRAPH_HEIGHT = 300;

    /**
     * Display the lists as a graph
     */
    private void display(ArrayList<Double> nums, ArrayList<Double> averages) {
        UI.clearGraphics();
        double min = Double.POSITIVE_INFINITY;
        double max = Double.NEGATIVE_INFINITY;
        for(Double d : nums) {
            min = Math.min(min, d);
            max = Math.max(max, d);
        }
        // Adjust max and min to push graph away from edges
        max = max + (max - min)*0.1;
        min = min - (max - min)*0.1;
        double yStep = GRAPH_HEIGHT / (max-min);
        double xStep = GRAPH_WIDTH / nums.size();
        // Draw axes:
        UI.setColor(Color.BLACK);
        UI.drawLine(GRAPH_LEFT, GRAPH_BASELINE, GRAPH_LEFT, GRAPH_BASELINE-GRAPH_HEIGHT);
        UI.drawLine(GRAPH_LEFT, GRAPH_BASELINE, GRAPH_LEFT+GRAPH_WIDTH, GRAPH_BASELINE);
        // Draw ticks and labels on x axis
        for(int i = 0; i < nums.size(); i++) {
            UI.drawLine(GRAPH_LEFT + xStep*i, GRAPH_BASELINE+5, GRAPH_LEFT + xStep*i, GRAPH_BASELINE-5);
            UI.drawString(String.format("%d",i), GRAPH_LEFT + xStep*i - 3, GRAPH_BASELINE+20);
        }
        // Draw ticks and labels on y axis
        for(double y = min; y <= max; y += (max-min)*0.10) {
            UI.drawLine(GRAPH_LEFT-5, GRAPH_BASELINE - (y - min)*yStep, GRAPH_LEFT + 5, GRAPH_BASELINE - (y - min)*yStep);
            UI.drawString(String.format("%.2f",y), GRAPH_LEFT - 30, GRAPH_BASELINE - (y - min)*yStep+5);
        }
        // Plot raw data
        UI.setColor(Color.RED);
        for (int i=1; i < nums.size(); i++) {
            UI.drawLine( GRAPH_LEFT + (i-1) * xStep,
                GRAPH_BASELINE - (nums.get(i-1)-min)*yStep,
                GRAPH_LEFT + (i) * xStep,
                GRAPH_BASELINE - (nums.get(i)-min)*yStep);
        }
        // Plot moving average
        UI.setColor(Color.BLUE);
        for (int i=1; i < averages.size(); i++) {
            UI.drawLine( GRAPH_LEFT + (i) * xStep,
                GRAPH_BASELINE - (averages.get(i-1)-min)*yStep,
                GRAPH_LEFT + (i+1) * xStep,
                GRAPH_BASELINE - (averages.get(i)-min)*yStep);
        }
    }

    public void setupGUI() {
        UI.initialise();
        UI.addButton("Run", this::doRun);
        UI.addTextField("", this::doFindMovingAverage);
        UI.addButton("Quit", UI::quit);
        UI.setWindowSize(700,500);
        UI.setDivider(0.3);
    }

    public static void main(String[] args) {
        new Program3().setupGUI();
    }
}
