// This program is copyright VUW.
// You are granted permission to use it to construct your answer to a COMP102 Lab Assessment.
// You may not distribute it in any other way without permission.

/* Code for COMP102 - 2020T3, Lab Assessment 3
 * Name:
 * Username:
 * ID:
 */

import ecs100.*;

import java.awt.FontMetrics;
import java.util.*;

/**
 * Question 5. 2D Arrays                                [23 marks]
 * 
 * This class contains a program for two highly simplified versions of a grid-based
 * logic puzzle. The program displays a grid of filled and unfilled squares, and an array of
 * numbers on the left that indicates the number of filled squares that should be in each row.
 * The user can click on the grid to toggle a square between filled and unfilled.
 * 
 * The 'grid' field is declared to hold a 2D array of booleans. 
 *  - A true value indicates that the square is filled in
 *  - a false value indicates that the square is blank.
 * 
 * The 'rowCounts' field is a 1D array that represents the number of filled squares that
 * appear in each row, from the top down.
 */
public class Program5 {

    private boolean[][] grid = new boolean[5][5];
    private int[] rowCounts = new int[5];

    // Constants
    public final int TOP = 50;
    public final int LEFT = 50;
    public final int SIZE = 20;
    public final int GRID_LEFT = LEFT + SIZE + 5;
    public final int GRID_TOP = TOP;

    /**
     * (a) [8 marks] Complete the checkRowCountsSimple() method so that it counts
     *   the number of filled squares in each row, and checks that the counts match
     *   the values stored in rowCounts. Your method should still work whatever the
     *   size of the grid array
     *
     *   - Filled squares do not need to be next to each other.
     *     If the count for a row is 3, all of the following are valid:
     *
     *     [X][X][X][ ][ ] or [X][X][ ][X][ ] or [X][ ][X][ ][X]
     *
     * @return true if all the rows are valid, false otherwise
     */
    public boolean checkRowsSimple() {
        /*# YOUR CODE HERE */

        return true;
    }

    /**
     * (b) [15 marks] Complete the checkRowsContinuous() method so that it counts
     *   the number of *connected* filled squares in each row, and checks that
     *   the counts match the values stored in rowCounts. Your method should still
     *   work whatever the size of the grid array.
     *
     *   - Filled squares must be next to each other.
     *     If the count for a row is 3, all of the following are valid:
     *
     *     [X][X][X][ ][ ] or [ ][X][X][X][ ] or [ ][ ][X][X][X]
     *
     *     The following are all INVALID as the squares aren't next to each other
     *
     *     [X][X][X][ ][ ] or [X][X][ ][X][ ] or [X][ ][X][ ][X]
     *
     * @return true if all the rows are valid, false otherwise
     */
    public boolean checkRowsContinuous() {
        /*# YOUR CODE HERE */

        return true;
    }

    /*********************************************
     * YOU CAN IGNORE EVERYTHING BELOW THIS LINE *
     *********************************************/

    /**
     * Creates a new random grid.
     * The grid will be 4-6 squares in size, and the row counts
     * will be random (but will always fit in the grid).
     */
    public void makeNewGrid() {
        Random r = new Random();
        int gridSize = 4 + r.nextInt(3);
        grid = new boolean[gridSize][gridSize];
        rowCounts = new int[gridSize];
        for (int i = 0; i < gridSize; i++) {
            rowCounts[i] = r.nextInt(gridSize+1);
        }
        drawGrid();
    }

    /**
     * Draws the grid on the screen.
     */
    public void drawGrid() {
        UI.clearGraphics();
        /*
         * This section ensures that the numbers are centered in their boxes. You do not
         * need to understand this (but I think it's pretty neat).
         */
        for (int r = 0; r < rowCounts.length; r++) {
            // Draw the box the number will go in
            UI.drawRect(LEFT, TOP + r * SIZE, SIZE, SIZE);
            // Generate the text (this is just the number).
            String text = "" + rowCounts[r];
            // Get the font metrics so we can figure out how big our text actually is
            FontMetrics metrics = UI.getGraphics().getFontMetrics(UI.getGraphics().getFont());
            // Get the width of the number in the selected font, and center it
            int x = LEFT + (SIZE - metrics.stringWidth(text)) / 2;
            // Get the height of the number in the selected font, and center it
            int y = TOP+r*SIZE + ((SIZE - metrics.getHeight()) / 2) + metrics.getAscent();
            UI.drawString(text, x, y);
        }
        // Draw the grid.
        for (int r = 0; r < grid.length; r++) {
            for (int c = 0; c < grid[r].length; c++) {
                // The filled rects always end up a pixel too small, so we
                // always draw the outline even if it's filled.
                UI.drawRect(GRID_LEFT + SIZE * c, GRID_TOP + SIZE * r, SIZE, SIZE);
                // Then, if the square is filled, draw a filled rect
                if (grid[r][c]) {
                    UI.fillRect(GRID_LEFT + SIZE * c, GRID_TOP + SIZE * r, SIZE, SIZE);
                }
            }
        }
    }

    /**
     *  Handle the mouse events
     */
    private void doMouse(String s, double x, double y) {
        // Ignore any event that isn't the user pressing the mouse button
        if (!s.equals("pressed")) {
            return;
        }
        // Make sure the user has clicked inside the grid
        if (x <= GRID_LEFT || x >= GRID_LEFT + grid.length * SIZE) {
            return;
        }
        if (y <= TOP || y >= TOP + grid[0].length * SIZE) {
            return;
        }
        // Turn screen positions (x,y) into grid positions (row,col)
        int row = (int) ((y - GRID_TOP) / SIZE);
        int col = (int) ((x - GRID_LEFT) / SIZE);
        //Flip the current value (true -> false; false -> true)
        grid[row][col] = !grid[row][col];
        //Update screen
        drawGrid();
    }

    /**
     * Run checkRowsSimple() and output the result.
     */
    public void doCheckRowsSimple() {
        if (checkRowsSimple()) {
            UI.println("Rows are valid.");
        } else {
            UI.println("Rows are not valid.");
        }
    }

    /**
     * Run checkRowsContinuous() and output the result.
     */
    public void doCheckRowsContinuous() {
        if (checkRowsContinuous()) {
            UI.println("Rows are valid.");
        } else {
            UI.println("Rows are not valid.");
        }
    }

    // Setup the GUI
    public void setupGUI() {
        UI.initialise();
        UI.addButton("Make New Grid", this::makeNewGrid);
        UI.addTextField("", this::doTextField);
        UI.addButton("Check Rows (Simple)", this::doCheckRowsSimple);
        UI.addButton("Check Rows (Continuous)", this::doCheckRowsContinuous);
        UI.setMouseListener(this::doMouse);
        UI.setWindowSize(800, 500);
        UI.setDivider(0.25);
        makeNewGrid();
    }

    // Handle the text field
    private void doTextField(String s) {
        Scanner scan = new Scanner(s);
        ArrayList<Integer> counts = new ArrayList<>();
        while(scan.hasNextInt()) {
            counts.add(scan.nextInt());
        }
        if (counts.size() > 0) {
            grid = new boolean[counts.size()][counts.size()];
            rowCounts = new int[counts.size()];
            for(int i=0;i<rowCounts.length;i++) {
                rowCounts[i] = counts.get(i);
            }
        } else {
            UI.println("Error creating grid.");
        }
        drawGrid();
    }

    // Run this
    public static void main(String[] args) {
        new Program5().setupGUI();
    }
}
